@@ -28,6 +28,7 @@ Boston, MA 02111-1307, USA.
 #include "filenaming.h"
 #include "prefs.h"
 #include "util.h"
+#include "metadata.h"
 
 gchar *chomp_strings[] =
 {
@@ -130,9 +131,9 @@ gchar *compose_filename(metadata_t *meta)
   return returnname;
 }
 
-static gint number_of_dashes(gchar *string)
+static gint number_of_dashes(const gchar *string)
 {
-  gchar *tmp;
+  const gchar *tmp;
   gint no = 0;
   
   tmp = string;
@@ -145,7 +146,7 @@ static gint number_of_dashes(gchar *string)
   return no;
 }
 
-static gchar *kill_tracknumber(gchar *string)
+static gchar *kill_tracknumber(const gchar *string)
 {
   /* If something beginning with two figures is followed
    * by something that is not a figure, we assume it is
@@ -155,18 +156,13 @@ static gchar *kill_tracknumber(gchar *string)
 
   if (strlen(string) < 4)
     return g_strdup(string);
-  test[0] = string[0];
-  test[1] = string[1];
-  test[2] = '\0';
-  if (is_a_number(test)) {
-    test[0] = string[2];
-    test[1] = '\0';
-    if (!is_a_number(test)) {
-      tmp = g_strdup(string+2);
-      g_strstrip(tmp);
-      g_free(string);
-      return tmp;
-    }
+  if (g_ascii_isdigit(string[0]) &&
+      g_ascii_isdigit(string[1]) &&
+      !g_ascii_isdigit(string[2]) &&
+      string[2] != '\'') { // This last condition is for year albums like "70's".
+    tmp = g_strdup(string+2);
+    g_strstrip(tmp);
+    return tmp;
   }
   return g_strdup(string);
 }
@@ -417,6 +413,9 @@ static gchar *get_artist (gchar *dir, gchar *file)
   gchar **tmp;
   gchar *artist = NULL;
 
+  if (gnomad_debug != 0) {
+    g_print("get_artist()\n");
+  }
   tmp = g_strsplit(dir, " - ", 0);
   /* If the directory is named "Foo - Bar", we presume
    * Foo is the artist, and Bar is the album title */
@@ -456,24 +455,57 @@ static gchar *get_title (gchar *file)
 {
   gchar **tmp;
   gchar *title = NULL;
+  gchar *file2;
+
+  if (gnomad_debug != 0) {
+    g_print("get_title()\n");
+  }
+
+  file2 = kill_tracknumber(file);
+  if (file2 == NULL) {
+    return NULL;
+  }
+  tmp = g_strsplit(file, " - ", 0);
+  g_free(file2);
+
+  if (vectorlength(tmp) > 1) {
+    gchar **tmp2 = tmp;
+    gchar *tmp3;
+
+    while(*tmp2 != NULL) {
+      tmp3 = *tmp2;
+      tmp2++;
+    }
+    g_print("%s\n", tmp3);
+  }
+  g_strfreev(tmp);
+  return g_strdup(file);
 
-  gchar *file2 = kill_tracknumber(file);
   tmp = g_strsplit(file2, " - ", 0);
-  /* If the file is named "F - oo - Bar", we assume
-   * Bar is the title */
-  if (vectorlength(tmp) > 2) {
-    title = g_strdup(*(tmp+2));
-  /* Else if the file is named "Foo - Bar", we assume
-   * Bar is the title */
-  } else if (vectorlength(tmp) > 1) {
-    title = g_strdup(*(tmp+1));
+  g_free(file2);
+  if (vectorlength(tmp) > 1) {
+    gchar **tmp2 = tmp;
+    gchar *tmp3;
+
+    while(*tmp2 != NULL) {
+      tmp3 = *tmp2;
+      tmp2++;
+    }
+    title = g_strdup(tmp3);
   } else {
     /* Otherwise we will use the file name as it is,
      * assuming this is the title is the best we can do... */
     title = g_strdup(file);
   }
+  // This is the culprit
   g_strfreev(tmp);
-  g_free(file2);
+  if (gnomad_debug != 0) {
+    if (title != NULL) {
+      g_print("Found title \"%s\"...\n", title);
+    } else {
+      g_print("Didn't find any title.\n");
+    }
+  }
   return title;
 }
 
@@ -482,6 +514,9 @@ static gchar *get_album (gchar *dir)
   gchar **tmp;
   gchar *album = NULL;
 
+  if (gnomad_debug != 0) {
+    g_print("get_album()\n");
+  }
   tmp = g_strsplit(dir, " - ", 0);
   /* If the directory is named "Foo - Bar", we presume
    * Foo is the artist, and Bar is the album title */
@@ -496,24 +531,29 @@ static guint get_trackno(gchar *file)
   /* If something beginning with two figures is followed
    * by something that is not a figure, we assume it is
    * a track number. */
-  gchar *p;
   gchar test[3];
-  gint found=0;
+  guint i;
 
-  memset(test, 0, sizeof(test));
+  if (gnomad_debug != 0) {
+    g_print("get_trackno()\n");
+  }
 
-  for (p=file; *p; ++p)
-    if (g_ascii_isdigit(*p)) {
-      if (found < 2)
-        test[found]=*p;
-      ++found;
-      if (found==2) {
-        if (!isdigit(p[1]))
-          return string_to_guint(test);
+  if (strlen(file) < 3) {
+    return 0;
+  }
+  for (i = 0; i < (strlen(file) - 2); i++) {
+    if (g_ascii_isdigit(file[i]) &&
+	g_ascii_isdigit(file[i+1]) &&
+	!g_ascii_isdigit(file[i+2])) {
+      test[0] = file[i];
+      test[1] = file[i+1];
+      test[2] = '\0';
+      if (gnomad_debug != 0) {
+	g_print("Found track number: %s\n", test);
       }
-    } else
-      found=0;
-
+      return string_to_guint(test);
+    }
+  }
   return 0;
 }
 
@@ -530,14 +570,18 @@ void info_from_path(metadata_t *meta, gboolean be_clever)
   guint track = 0;
   int i;
 
-  /* g_print("Getting info from path...\n"); */
+  if (gnomad_debug != 0) {
+    g_print("Getting info from path...\n");
+  }
   /* If ID3lib has already set seriously acceptable tags,
    * use them and return */
   if (meta->artist != NULL &&
       meta->title != NULL &&
       meta->album != NULL &&
       meta->trackno > 0) {
-    /* g_print("All metadata already set.\n"); */
+    if (gnomad_debug != 0) {
+      g_print("All metadata already set.\n");
+    }
     return;
   }
   /* It's not much use trying to determine something
@@ -550,7 +594,9 @@ void info_from_path(metadata_t *meta, gboolean be_clever)
    * cook up Artist / Title / Album fields from the path.
    */
   path = g_strdup(meta->path);
-  //g_print("Processing path: %s\n", path);
+  if (gnomad_debug != 0) {
+    g_print("Processing path: %s\n", path);
+  }
   /* Chop off the file extension */
   if (path[strlen(path)-3] == '.')
     path[strlen(path)-3] = '\0';
@@ -576,8 +622,10 @@ void info_from_path(metadata_t *meta, gboolean be_clever)
     tmpdir = beautify_string(tmpdir);
     tmpfile = beautify_string(tmpfile);
   }
-  //g_print("Dir: %s\n", tmpdir);
-  //g_print("File: %s\n", tmpfile);
+  if (gnomad_debug != 0) {
+    g_print("Dir: %s\n", tmpdir);
+    g_print("File: %s\n", tmpfile);
+  }
   // If we're not being too clever, do as mediasource do.
   if (!be_clever) {
     if ((meta->title == NULL) && (tmpfile != NULL)) {
@@ -600,4 +648,8 @@ void info_from_path(metadata_t *meta, gboolean be_clever)
     meta->trackno = get_trackno(tmpfile);
   g_free(tmpdir);
   g_free(tmpfile);
+  if (gnomad_debug != 0) {
+    g_print("Got all info from path...\n");
+    dump_metadata_t(meta);
+  }
 }
