@@ -147,8 +147,8 @@ gboolean is_directory(gchar *dir) {
 static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
 {
   GList *filelist;
-  DIR * DirPointer;
-  struct dirent * Entry;
+  DIR * dirp;
+  struct dirent * dir_entry;
   gchar *tmppath;
 
   /* Returns a listing of applicable files */
@@ -156,70 +156,78 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
   /* Directories: no full path, just dirname; size 0 */
   filelist = NULL;
   tmppath = filename_fromutf8(base_path);
-  DirPointer = opendir(tmppath);
-  if (! DirPointer){
-    /* g_print("Unable to open directory %s.\n", tmppath); */
+  dirp = opendir(tmppath);
+  if (!dirp){
+    if (gnomad_debug != 0) {
+      g_print("Unable to open directory %s.\n", tmppath);
+    }
     return NULL;
   }
-  while ((Entry = readdir(DirPointer)) != NULL)
+  while ((dir_entry = readdir(dirp)) != NULL)
     {
-      struct stat EntryStatus;
-      gchar cTempPath[512];
-      gchar *cTempFilename;
-      int nTempSize;
-      int bIsDir;
-      int bIsFile;
+      struct stat entry_status;
+      gchar path_buffer[512];
+      gchar *filename;
+      int size;
+      int is_dir;
+      int is_file;
 
-      strncpy(cTempPath, tmppath, 512);
-      strcat(cTempPath, "/");
-      strcat(cTempPath, Entry->d_name);
-      if (stat(cTempPath, &EntryStatus)) {
-        /* g_print("Failed to read file status for %s.\n", cTempPath); */
+      strncpy(path_buffer, tmppath, 512);
+      strcat(path_buffer, "/");
+      strcat(path_buffer, dir_entry->d_name);
+      if (stat(path_buffer, &entry_status)) {
+	if (gnomad_debug != 0) {
+	  g_print("Failed to read file status for %s.\n", path_buffer);
+	}
         continue;
       }
-      nTempSize = (int) EntryStatus.st_size;
-      bIsDir = S_ISDIR(EntryStatus.st_mode);
-      bIsFile = S_ISREG(EntryStatus.st_mode);
-      if (bIsDir) {
+      size = (int) entry_status.st_size;
+      is_dir = S_ISDIR(entry_status.st_mode);
+      is_file = S_ISREG(entry_status.st_mode);
+      if (is_dir) {
 	/* We don't want this "dir" */
-	if (!strcmp(Entry->d_name,".")) {
+	if (!strcmp(dir_entry->d_name,".")) {
 	  continue;
 	}
 	/* If the prefs are selected not to display the hidden
 	 * directories, we hide them. */
-	if (strlen(Entry->d_name) >= 2) {
-	  if (Entry->d_name[0] == '.' &&
-	      Entry->d_name[1] != '.' &&
+	if (strlen(dir_entry->d_name) >= 2) {
+	  if (dir_entry->d_name[0] == '.' &&
+	      dir_entry->d_name[1] != '.' &&
 	      !get_prefs_display_dotdirs())
 	    continue;
 	}
-      } else if (bIsFile) {
-	static char cTempCodec[4];
+      } else if (is_file) {
+	char tmp_codec[4];
 	int i;
-	
-	if (strlen(Entry->d_name) < 4)
-	  continue;
-	strcpy(cTempCodec, Entry->d_name+strlen(Entry->d_name)-3);
-	g_strdown(cTempCodec);
-	/* Only display valid files */
-	if (!anyfile &&
-	    g_ascii_strcasecmp(cTempCodec,"mp3") &&
-	    g_ascii_strcasecmp(cTempCodec,"wav") &&
-	    g_ascii_strcasecmp(cTempCodec,"wma"))
-	  continue;
+
+	if (!anyfile) {
+	  if (strlen(dir_entry->d_name) < 4)
+	    continue;
+	  strcpy(tmp_codec, dir_entry->d_name+strlen(dir_entry->d_name)-3);
+	  g_strdown(tmp_codec);
+	  /* Only display valid files */
+	  if (g_ascii_strcasecmp(tmp_codec,"mp3") &&
+	      g_ascii_strcasecmp(tmp_codec,"wav") &&
+	      g_ascii_strcasecmp(tmp_codec,"wma"))
+	    continue;
+	}
       }
       /* only directories and valid files come here, the
        * list is NOT in UTF-8, that conversion will be
        * done by the caller of this routine. */
-      if (bIsDir) {
+      if (is_dir) {
 	/* For directories, set size to 0 */
-	nTempSize = 0;
+	size = 0;
       }
-      cTempFilename = g_strdup(cTempPath);
-      filelist = g_list_append(filelist, cTempFilename);
-      filelist = g_list_append(filelist, GUINT_TO_POINTER(nTempSize));
+      filename = g_strdup(path_buffer);
+      if (gnomad_debug != 0) {
+	g_print("Adding dir entry: %s, size %d\n", filename, size);
+      }
+      filelist = g_list_append(filelist, filename);
+      filelist = g_list_append(filelist, GUINT_TO_POINTER(size));
     }
-  closedir(DirPointer);
+  closedir(dirp);
   g_free(tmppath);
   return filelist;
 }
@@ -248,7 +256,7 @@ static GList *get_metadata_for_dirlist(GList *inlist,
   }
   
   filelist = inlist;
-  while (filelist && !cancelled) {
+  while ((filelist != NULL) && !cancelled) {
     metadata_t *meta;
     gchar *path;
     gchar *data;
@@ -266,13 +274,21 @@ static GList *get_metadata_for_dirlist(GList *inlist,
     /* If filename is ISO-8859-1 convert it by force! */
     filelist = filelist->next;
     meta->size = GPOINTER_TO_UINT(filelist->data);
+    filelist = filelist->next;
     tmp = g_path_get_basename(path);
-    meta->filename = filename_toutf8(tmp);
-    g_free(tmp);
+    if (tmp == NULL) {
+      meta->filename = NULL;
+      g_print("Basename was NULL\n");
+    } else {
+      meta->filename = filename_toutf8(tmp);
+      g_free(tmp);
+    }
     if (use_dialog) {
       gdk_threads_enter();
       tmp = g_strdup_printf("File %d of %d", currentfile, nofiles);
-      /* g_print("File: %s\n", meta->filename); */
+      if (gnomad_debug != 0) {
+	g_print("Getting metadata for: %s (%d/%d)\n", meta->filename, currentfile, nofiles);
+      }
       currentfile ++;
       gtk_label_set_text(GTK_LABEL(args->label), tmp);
       gdk_threads_leave();
@@ -292,6 +308,9 @@ static GList *get_metadata_for_dirlist(GList *inlist,
 	meta->codec = g_strdup(codec);
 	if (!strcmp(meta->codec,"MP3")) {
 	  /* Gets the ID3 tag if any */
+	  if (gnomad_debug != 0) {
+	    g_print("Getting ID3 info for: %s\n", meta->filename);
+	  }
 	  get_tag_for_mp3file(meta);
 	  /* Scans the mp3 frames to calculate play length */
 	  if ((meta->length == NULL) && (meta->path != NULL)) {
@@ -368,7 +387,6 @@ static GList *get_metadata_for_dirlist(GList *inlist,
       gdk_threads_leave();
       cancelled = args->cancelled;
     }
-    filelist = filelist->next;
   }
   return metalist;
 }
