@@ -26,10 +26,23 @@ Boston, MA 02111-1307, USA.
 #define CREATE_FILEMODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
 #define CREATE_DIRMODE (S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
 
+/* Local type for the metadata read thread */
+typedef struct 
+{
+  GtkWidget *dialog;
+  GtkWidget *label;
+  GtkWidget *progress;
+  listtype_t listtype;
+  gchar *path;
+  gboolean cancelled;
+} dirfill_thread_arg_t;
+
 /* Exported functions */
 gchar *filename_toutf8(const gchar *instr);
 gchar *filename_fromutf8(const gchar *instr);
-GList *get_metadata_dir(listtype_t listtype, gchar *path);
+GList *get_metadata_dir(listtype_t listtype, 
+			gchar *path, 
+			dirfill_thread_arg_t *args);
 void fill_in_dir(listtype_t listtype, gchar *path);
 gchar *get_current_dir();
 gboolean is_directory(gchar *dir);
