@@ -94,6 +94,7 @@ static gboolean hasid3tag(gchar *path)
   return result;
 }
 
+/*
 static gboolean addid3tag(gchar *path)
 {
   // File descriptors...
@@ -217,6 +218,7 @@ static gboolean addid3tag(gchar *path)
   close(f1);
   return FALSE;
 }
+*/
 
 /*
  * This function checks if a RIFF header exists at the beginning of 
@@ -641,14 +643,8 @@ getOrigFilename(struct id3_tag *tag)
   return getFrameText(tag, "TOFN");
 }
 
-
-/*****************************************************************************
- * EXPORTED FUNCTIONS
- *****************************************************************************/
-
-
-void
-remove_tag_from_mp3file(gchar *path)
+static void
+remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guchar *v1tag, guint v1taglen)
 {
   // File descriptors...
   gint f1, f2, f3;
@@ -666,7 +662,9 @@ remove_tag_from_mp3file(gchar *path)
   template[strlen(template)] = G_DIR_SEPARATOR;
   strcat(template, "gnomadXXXXXX");
 
-  // g_print("Removing ID3 tags from %s\n", path);
+  if (gnomad_debug != 0) {
+    g_print("Removing or adding ID3 tags from/to %s\n", path);
+  }
 
   f1 = (gint) open(path, O_RDONLY, 0);
   if (f1 < 0) {
@@ -685,6 +683,7 @@ remove_tag_from_mp3file(gchar *path)
   // Temporary file
   f2 = g_mkstemp(template);
   // g_print("Opened temporary file f2...\n");
+
   if (f2 >= 0) {
     guchar tag[10];
     gint n;
@@ -809,6 +808,11 @@ remove_tag_from_mp3file(gchar *path)
 
       // g_print("Header %d bytes, footer %d bytes to be removed.\n", header_taglength, footer_taglength);
 
+      if (add) {
+	// Write ID3v2 tag at the beginning of the file
+	write(f2,v2tag,v2taglen);
+      }
+
       // Next skip past the header, and copy until we reach the footer
       if (lseek(f1, header_taglength, SEEK_SET) >= 0) {
 	guint remain = filelength - header_taglength - footer_taglength;
@@ -835,6 +839,11 @@ remove_tag_from_mp3file(gchar *path)
     }
     close(f1);
 
+    if (add) {
+      // Write ID3v1 tag at the end of the file
+      write(f2,v1tag,v1taglen);
+    }
+
     // Then copy back the stripped file
     // Rewind the temporary file
     lseek(f2, 0, SEEK_SET);
@@ -872,6 +881,20 @@ remove_tag_from_mp3file(gchar *path)
   g_free(buffer);
 }
 
+
+
+/*****************************************************************************
+ * EXPORTED FUNCTIONS
+ *****************************************************************************/
+
+
+void
+remove_tag_from_mp3file(gchar *path)
+{
+  remove_or_add_tag(FALSE, path, NULL, 0, NULL, 0);
+}
+
+
 void
 get_tag_for_mp3file (metadata_t *meta)
 {
@@ -919,6 +942,11 @@ get_tag_for_mp3file (metadata_t *meta)
   id3_file_close(fh);
 }
 
+/*
+ * TODO: change this function to always strip the current tag
+ * and add a new one by rendering both v1 and v2 tags to a buffer
+ * and concatenate with the file.
+ */
 void
 set_tag_for_mp3file (metadata_t *meta, gboolean override)
 {
@@ -926,15 +954,18 @@ set_tag_for_mp3file (metadata_t *meta, gboolean override)
   struct id3_tag *tag;
   gchar *tmppath = filename_fromutf8(meta->path);
   enum id3_field_textencoding encoding;
-
-
-  // g_print("Setting tag info for %s...\n", tmppath);
-  /* First check that the file has some kind of tag, else add it! */
-  if (!hasid3tag(tmppath)) {
-    addid3tag(tmppath);
+  id3_length_t tagv2len;
+  id3_length_t tagv1len;
+  id3_byte_t *tagv2;
+  id3_byte_t *tagv1;  
+  
+  if (gnomad_debug != 0) {
+    g_print("Setting tag info for %s...\n", tmppath);
   }
+
+  /* Get the tag for the old file */
   fh = id3_file_open(tmppath, ID3_FILE_MODE_READWRITE);
-  g_free(tmppath);
+
   if (!fh) {
     g_print("Could not open file %s!\n", meta->path);
     return;
@@ -945,13 +976,16 @@ set_tag_for_mp3file (metadata_t *meta, gboolean override)
     return;
   }
 
-  /* use the same coding as before... */
+  /* use the same encoding as before... */
   encoding = get_encoding_of_tag(tag);
   /* ...unless it's ISO_8859_1 and prefs say we should use
      unicode (i.e. ID3v2.4) */
   if (get_prefs_useid3_unicode() && (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1))
     encoding = ID3_FIELD_TEXTENCODING_UTF_8;
- 
+
+  /* Close old file. */
+  // id3_file_close(fh);
+
   if (meta->artist != NULL &&
       strcmp(meta->artist, "<Unknown>")) {
     setArtist(tag, meta->artist, encoding, override);
@@ -985,8 +1019,34 @@ set_tag_for_mp3file (metadata_t *meta, gboolean override)
       strcmp(meta->filename, "0")) {
     setOrigFilename(tag, meta->filename, encoding, override);
   }
-  if (id3_file_update(fh) != 0) {
-    g_print("ERROR while writing tag to file: '%s'.\n", meta->path);
+
+  /* Render tag so we can look at it */
+  tagv2 = g_malloc(64738);
+  tagv1 = g_malloc(128);
+
+  /* Render ID3v2 tag */
+  id3_tag_options(tag, ID3_TAG_OPTION_ID3V1, 0);
+  tagv2len = id3_tag_render(tag, tagv2);
+  if (gnomad_debug != 0) {
+    g_print("Rendered ID3v2 tag:\n");
+    hexdump(tagv2, tagv2len);
   }
+
+  /* Render ID3v1 tag */
+  id3_tag_options(tag, ID3_TAG_OPTION_ID3V1, ~0);
+  tagv1len = id3_tag_render(tag, tagv1);
+  if (gnomad_debug != 0) {
+    g_print("Rendered ID3v1 tag:\n");
+    hexdump(tagv1, tagv1len);
+  }
+
+  /* Close old file */
   id3_file_close(fh);
+
+  /* Totally revamp file */
+  remove_or_add_tag(TRUE, tmppath, tagv2, tagv2len, tagv1, tagv1len);
+
+  g_free(tagv2);
+  g_free(tagv1);
+  g_free(tmppath);
 }
