@@ -721,6 +721,10 @@ void view_and_sort_list_store(listtype_t listtype)
 {
   gint i;
 
+  if (gnomad_debug != 0) {
+    g_print("Enter view_and_sort_list_store(%d)...\n", (gint) listtype);
+  }
+
   /* All should be found now */
   gtk_tree_view_set_model (GTK_TREE_VIEW (get_view(listtype)),
                            GTK_TREE_MODEL (get_store(listtype)));
